@@ -1,22 +1,8 @@
-/********************************************************************
-Copyright 2014  Martin Gräßlin <mgraesslin@kde.org>
+/*
+    SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) version 3, or any
-later version accepted by the membership of KDE e.V. (or its
-successor approved by the membership of KDE e.V.), which shall
-act as a proxy defined in Section 6 of version 3 of the license.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 #include "surface_interface.h"
 #include "surface_interface_p.h"
 #include "buffer_interface.h"
@@ -560,10 +546,6 @@ void SurfaceInterface::Private::damage(const QRect &rect)
 
 void SurfaceInterface::Private::damageBuffer(const QRect &rect)
 {
-    if (!pending.bufferIsSet || (pending.bufferIsSet && !pending.buffer)) {
-        // TODO: should we send an error?
-        return;
-    }
     pending.bufferDamage = pending.bufferDamage.united(rect);
 }
 
@@ -786,6 +768,19 @@ QSize SurfaceInterface::size() const
     return QSize();
 }
 
+QRect SurfaceInterface::boundingRect() const
+{
+    QRect rect(QPoint(0, 0), size());
+
+    const QList<QPointer<SubSurfaceInterface>> subSurfaces = childSubSurfaces();
+    for (const SubSurfaceInterface *subSurface : subSurfaces) {
+        const SurfaceInterface *childSurface = subSurface->surface();
+        rect |= childSurface->boundingRect().translated(subSurface->position());
+    }
+
+    return rect;
+}
+
 QPointer< ShadowInterface > SurfaceInterface::shadow() const
 {
     Q_D();
